﻿//Common Bind Module Method
var bindModule = function (thisModule, controllerProvider, compileProvider, filterProvider, provide) {

    thisModule.controller = controllerProvider.register;
    thisModule.directive = compileProvider.directive;
    thisModule.filter = filterProvider.register;
    thisModule.factory = provide.factory;
    thisModule.service = provide.service;

    return thisModule;
};

var invoiceModule = angular.module('app.invoice', ['ui.grid', 'ui.grid.selection', 'ui.grid.moveColumns', 'ui.grid.grouping', 'ui.grid.expandable', 'ui.grid.treeView', 'ui.grid.exporter', 'ui.grid.resizeColumns', 'mc.resizer', 'ui.select', 'ngSanitize', 'ui.grid.edit', 'perfect_scrollbar', 'dx', 'ui.grid.pinning'])
    .run(['$log', function ($log) {
        $log.debug('app.invoice.run()...');
    }])
    .config(['$controllerProvider', '$urlRouterProvider', '$compileProvider', '$filterProvider', '$provide', '$stateProvider', 'uiSelectConfig', 'scriptDependencyProvider',
        function ($controllerProvider, $urlRouterProvider, $compileProvider, $filterProvider, $provide, $stateProvider, uiSelectConfig, scriptDependencyProvider) {
            'use strict';
            // this is required to add controller/directive/filter/service after angular bootstrap
            bindModule(invoiceModule, $controllerProvider, $compileProvider, $filterProvider, $provide);

            uiSelectConfig.theme = 'select2';

            //发票管理框架 
            $stateProvider.state({
                name: 'invoiceManagement',
                url: '/invoiceManagement',
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation', '$state',
                                        function ($scope, $stateParams, appTranslation, $state) {
                                            appTranslation.load([appTranslation.vat]);
                                            $scope.state = $state;
                                        }],
                        template: '<invoice-manage-infrastructure state="state"></invoice-manage-infrastructure>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });

            //发票管理主要界面
            $stateProvider.state('invoiceManagement.main', {
                url: "/main",
                sticky: true,
                dsr: true,
                views: {
                    'main-view@invoiceManagement': {
                        template: '<invoice-manage-main></invoice-manage-main>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            //发票管理上传index
            $stateProvider.state('invoiceManagement.uploadIndex', {
                url: "/main/uploadIndex",
                sticky: true,
                dsr: true,
                views: {
                    'uploadIndex-view@invoiceManagement': {

                        template: '<invoice-manage-main-upload-index></invoice-manage-main-upload-index>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });
            //发票管理上传paper invoice
            $stateProvider.state('invoiceManagement.uploadPaperInvoice', {
                url: "/main/uploadPaperInvoice",
                sticky: true,
                dsr: true,
                views: {
                    'uploadPaperInvoice-view@invoiceManagement': {

                        template: '<invoice-manage-main-upload-paper-invoice></invoice-manage-main-upload-paper-invoice>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            //发票管理上传electonic invoice
            $stateProvider.state('invoiceManagement.uploadElecInvoice', {
                url: "/main/uploadElecInvoice",
                sticky: true,
                dsr: true,
                views: {
                    'uploadElecInvoice-view@invoiceManagement': {

                        template: '<invoice-manage-main-upload-electronic-invoice></invoice-manage-main-upload-electronic-invoice>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            //发票管理上传scan invoice
            $stateProvider.state('invoiceManagement.uploadScanInvoice', {
                url: "/main/uploadScanInvoice",
                sticky: true,
                dsr: true,
                views: {
                    'uploadScanInvoice-view@invoiceManagement': {

                        template: '<invoice-manage-main-upload-scan-invoice></invoice-manage-main-upload-scan-invoice>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });


            //发票管理未收票认证
            $stateProvider.state('invoiceManagement.noInvoiceVerify', {
                url: "/noInvoiceVerify",
                sticky: true,
                dsr: true,
                views: {
                    'noInvoiceVerify-view@invoiceManagement': {
                        template: '<invoice-manage-verify></invoice-manage-verify>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });


            //开票管理
            $stateProvider.state('invoiceManagement.billingManage',
            {
                url: "/billingManage",
                sticky: true,
                dsr: true,
                views: {
                    'billingManage-view@invoiceManagement': {
                        template: '<iframe id="dfiframe" src="/app/df/code/pages/PreViewFapiao.html" scrolling="auto" height="100%" style="border:none" width="100%"></iframe>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });



            //发票查询
            $stateProvider.state('invoiceManagement.invoiceQuery',
            {
                url: "/invoiceQuery",
                sticky: true,
                dsr: true,
                views: {
                    'invoiceQuery-view@invoiceManagement': {
                        template: '<iframe id="dfiframe" src="/app/df/code/pages/FapiaoInqury.html" scrolling="auto" height="100%" style="border:none" width="100%"></iframe>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });


            //发票管理待退票管理
            $stateProvider.state('invoiceManagement.pendingRefundInvoice', {
                url: "/pendingRefundInvoice",
                sticky: true,
                dsr: true,
                views: {
                    'pendingRefundInvoice-view@invoiceManagement': {
                        template: '<invoice-manage-refund></invoice-manage-refund>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            //发票管理关联管理
            $stateProvider.state('invoiceManagement.mapInvoice', {
                url: "/mapInvoice",
                sticky: true,
                dsr: true,
                views: {
                    'mapInvoice-view@invoiceManagement': {
                        template: '<invoice-manage-map></invoice-manage-map>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });


            //发票管理框架 
            $stateProvider.state({
                name: 'outputInvoiceManage',
                url: '/outputInvoiceManage',
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation', '$state',
                                        function ($scope, $stateParams, appTranslation, $state) {
                                            appTranslation.load([appTranslation.vat]);
                                            $scope.state = $state;
                                        }],
                        template: '<output-invoice-infrastructure state="state"></output-invoice-infrastructure>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });

            //发票管理主要界面
            $stateProvider.state('outputInvoiceManage.imported', {
                url: "/imported",
                sticky: false,
                dsr: true,
                views: {
                    'outputInvoiceImported-view@outputInvoiceManage': {
                        template: '<output-invoice-imported></output-invoice-imported>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            //发票管理编辑界面
            $stateProvider.state('outputInvoiceManage.outputInvoiceEdit', {
                url: "/outputInvoiceEdit",
                sticky: false,
                dsr: true,
                views: {
                    'outputInvoiceEdit-view@outputInvoiceManage': {
                        template: '<output-invoice-edit></output-invoice-edit>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            //发票管理已打印BD-USER发票
            $stateProvider.state('outputInvoiceManage.issuedInvoiceBdView', {
                url: "/issuedInvoiceBdView",
                sticky: false,
                dsr: true,
                views: {
                    'issuedInvoiceBd-view@outputInvoiceManage': {
                        template: '<issued-invoice-bd-view></issued-invoice-bd-view>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            $stateProvider.state('outputInvoiceManage.bdViewByVin', {
                url: "/bdViewByVin",
                sticky: false,
                dsr: true,
                views: {
                    'bdViewByVin-view@outputInvoiceManage': {
                        template: '<bd-view-by-vin></bd-view-by-vin>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });


            //GD已开具发票特殊处理
            $stateProvider.state('outputInvoiceManage.issuedInvoiceSpecial', {
                url: "/issuedInvoiceSpecial",
                sticky: false,
                dsr: true,
                views: {
                    'issuedInvoiceSpecial-view@outputInvoiceManage': {
                        template: '<special-vat-invoice></special-vat-invoice>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });
            //GD已开具发票特殊处理
            $stateProvider.state('outputInvoiceManage.GDViewByVIN', {
                url: "/GDViewByVIN",
                sticky: false,
                dsr: true,
                views: {
                    'GDViewByVIN-view@outputInvoiceManage': {
                        template: '<gd-view-by-vin></gd-view-by-vin>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            $stateProvider.state('outputInvoiceManage.outputReconciliationOverview', {
                url: "/outputReconciliationOverview",
                sticky: false,
                dsr: true,
                views: {
                    'output-reconciliation-overview@outputInvoiceManage': {
                        template: '<output-reconciliation-overview></output-reconciliation-overview>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            $stateProvider.state('outputInvoiceManage.reconciliationRuleConfig', {
                url: "/reconciliationRuleConfig",
                sticky: false,
                dsr: true,
                views: {
                    'reconciliation-rule-config@outputInvoiceManage': {
                        template: '<reconciliation-rule-config></reconciliation-rule-config>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            $stateProvider.state('outputInvoiceManage.outputReconciliationConfiguration', {
                url: "/outputReconciliationConfiguration",
                sticky: false,
                dsr: true,
                views: {
                    'output-reconciliation-configuration@outputInvoiceManage': {
                        template: '<output-reconciliation-configuration></output-reconciliation-configuration>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            $stateProvider.state('outputInvoiceManage.taxControlDisk', {
                url: "/taxControlDisk",
                sticky: false,
                dsr: true,
                views: {
                    'tax-control-disk@outputInvoiceManage': {
                        template: '<tax-control-disk></tax-control-disk>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

            ////GD已开具发票详情
            //$stateProvider.state('outputInvoiceManage.issuedInvoiceSpecialDetail', {
            //    url: "/issuedInvoiceSpecialDetail/{gdid}/{parentPage}",
            //    sticky: true,
            //    dsr: true,
            //    //params: {
            //    //    GDID: null,
            //    //    parentPage:null
            //    //},              
            //    views: {
            //        'issuedInvoiceSpecialDetail-view@outputInvoiceManage': {
            //            controller: ['$scope', '$stateParams', 'appTranslation',
            //                          function ($scope, $stateParams, appTranslation) {
            //                              console.log($stateParams.gdid);
            //                              $scope.gdInvoiceID = $stateParams.gdid;
            //                              $scope.parentPage = $stateParams.parentPage;
            //                          }],
            //            template: '<invoice-detail-vehicle></invoice-detail-vehicle>'

            //        }
            //    },
            //    resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            //});
            //发票管理已打印GD-BD发票
            $stateProvider.state('outputInvoiceManage.issuedInvoiceGdView', {
                url: "/issuedInvoiceGdView",
                sticky: false,
                dsr: true,
                views: {
                    'issuedInvoiceGd-view@outputInvoiceManage': {
                        template: '<issued-invoice-gd-view></issued-invoice-gd-view>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });



            //发票管理编辑界面
            $stateProvider.state('outputInvoiceManage.gdInvoicePrint', {
                url: "/gdInvoicePrint",
                sticky: false,
                params: { "invoiceList": null },
                dsr: true,
                views: {
                    'gdInvoicePrint-view@outputInvoiceManage': {
                        template: '<gd-invoice-print></gd-invoice-print>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });


            //发票管理编辑界面
            $stateProvider.state('outputInvoiceManage.importShipmentList', {
                url: "/importShipmentList",
                sticky: false,
                dsr: true,
                views: {
                    'importShipment-view@outputInvoiceManage': {
                        template: '<import-shipment-list></import-shipment-list>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat)
            });

        }]);

var citModule = angular.module('app.cit', ['ui.grid', 'ui.grid.selection', 'ui.grid.moveColumns', 'ui.grid.grouping', 'ui.grid.expandable', 'ui.grid.treeView', 'ui.grid.exporter', 'ui.grid.resizeColumns', 'mc.resizer', 'ui.select', 'ngAnimate', 'ngSanitize', 'ui.bootstrap', 'ui.grid.edit', 'perfect_scrollbar', 'dx', 'ui.grid.pinning','ngFileSaver'])
    .run(['$log', function ($log) {
        $log.debug('app.cit.run()...');
    }])
    .config(['$controllerProvider', '$urlRouterProvider', '$compileProvider', '$filterProvider', '$provide', '$stateProvider', 'uiSelectConfig', 'scriptDependencyProvider',
        function ($controllerProvider, $urlRouterProvider, $compileProvider, $filterProvider, $provide, $stateProvider, uiSelectConfig, scriptDependencyProvider) {
            'use strict';
            // this is required to add controller/directive/filter/service after angular bootstrap
            bindModule(citModule, $controllerProvider, $compileProvider, $filterProvider, $provide);

            uiSelectConfig.theme = 'select2';

            // $urlRouterProvider.when('/cit/previewData', '/cit/previewData/balanceSheet');

            // $urlRouterProvider.when('/cit/reductionData', 'cit/reductionData/caculateData');

            // $urlRouterProvider.when('/cit/analyzeReport', 'cit/analyzeReport/citModelAnalysis');

            $stateProvider.state({
                name: 'cit',
                url: '/cit',
                cache: false,
                views: {
                    '@': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                            function ($scope, $stateParams, appTranslation) {
                                appTranslation.load([appTranslation.cit]);
                            }],
                        template: '<cit-layout></cit-layout>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData',
                url: '/importData',
                views: {
                    '@cit': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-layout></cit-import-layout>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });


            $stateProvider.state({
                name: 'cit.importData.balanceSheet',
                url: '/balanceSheet',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-trial-balance></cit-import-trial-balance>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.assetList',
                url: '/assetList',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-asset-list></cit-import-asset-list>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            //demo资产清单页面
            $stateProvider.state({
                name: 'cit.importData.oldAssetList',
                url: '/oldAssetList',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-old-import-asset-list></cit-old-import-asset-list>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.documentList',
                url: '/documentList?{docId:string}&{docTemplateId:string}&{docName:string}&{treeTypeCode:string}&{selectedReportCode:string} \
                        &{hasDocument}&{docNum}',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            $scope.docId = $stateParams.docId;
                                            $scope.docTemplateId = $stateParams.docTemplateId;
                                            $scope.docName = $stateParams.docName;
                                            $scope.treeTypeCode = $stateParams.treeTypeCode;
                                            $scope.selectedReportCode = $stateParams.selectedReportCode;
                                            $scope.hasDocument = $stateParams.hasDocument;
                                            $scope.docNum = $stateParams.docNum;
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-document-list doc-id="docId" doc-template-id="docTemplateId" doc-name="docName" \
                                    tree-type-code="treeTypeCode" selected-report-code="selectedReportCode" has-document="hasDocument" doc-num="docNum"> \
                                    </cit-import-document-list>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: false,
                sticky: false
            });

            $stateProvider.state({
                name: 'cit.importData.documentList.reportView',
                url: '/reportView?{id:string}&{templateid:string}&{name:string}&{templatecode:string}&{initrow:any}&{initcol:any}&{parentid:string}',
                views: {
                    '@cit.importData.documentList': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            $scope.reportId = $stateParams.id;
                                            $scope.templateId = $stateParams.templateid;
                                            $scope.templateName = $stateParams.name;
                                            $scope.templateCode = $stateParams.templatecode;
                                            $scope.initRow = $stateParams.initrow;
                                            $scope.initCol = $stateParams.initcol;
                                            $scope.parentId = $stateParams.parentId;
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<vat-report-view report-id="reportId" template-id="templateId" template-name="templateName" template-code="templateCode" \
init-row="initRow" init-col="initCol" service-type="\'6\'" is-document-list="true" parent-id="parentId"></vat-report-view>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: false,
                sticky: false
            });

            $stateProvider.state({
                name: 'cit.importData.journalEntry',
                url: '/journalEntry',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-journal-entry></cit-import-journal-entry>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.erpData',
                url: '/erpData',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-erp></cit-import-erp>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.outputInvoice',
                url: '/outputInvoice',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-output-invoice></cit-import-output-invoice>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.inputInvoice',
                url: '/inputInvoice',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-income-invoice></cit-import-income-invoice>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.customInvoice',
                url: '/customInvoice',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-custom-invoice></cit-import-custom-invoice>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.voucherMapping',
                url: '/voucherMapping',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-voucher-mapping></cit-voucher-mapping>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.invoiceMapping',
                url: '/invoiceMapping',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-invoice-mapping></cit-invoice-mapping>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.auditAdjust',
                url: '/auditAdjust',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-audit-adjust></cit-audit-adjust>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.importData.importDataView',
                url: '/importDataView?{id:string}&{serviceType:string}',
                views: {
                    '@cit.importData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                            function ($scope, $stateParams, appTranslation) {
                                $scope.tableId = $stateParams.id;
                                $scope.serviceType = $stateParams.serviceType;
                                appTranslation.load([appTranslation.cit]);
                            }],
                        template: '<import-data-modal table-id="tableId" service-type="serviceType"></import-data-modal>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.vat),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.previewData',
                url: '/previewData',
                views: {
                    '@cit': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                                    function ($scope, $stateParams, appTranslation) {
                                                        appTranslation.load([appTranslation.cit]);
                                                    }],
                        template: '<cit-import-layout></cit-import-layout>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.previewData.balanceSheet',
                url: '/balanceSheet',
                views: {
                    '@cit.previewData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-preview-balance-sheet></cit-preview-balance-sheet>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.previewData.accountVoucher',
                url: '/accountVoucher',
                views: {
                    '@cit.previewData': {
                        controller: [
                            '$scope', '$stateParams', 'appTranslation',
                            function ($scope, $stateParams, appTranslation) {
                                appTranslation.load([appTranslation.cit]);
                            }
                        ],
                        template: '<cit-preview-account-voucher></cit-preview-account-voucher>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.previewData.outputInvoice',
                url: '/outputInvoice',
                views: {
                    '@cit.previewData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-preview-output-invoice></cit-preview-output-invoice>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.previewData.inputInvoice',
                url: '/inputInvoice',
                views: {
                    '@cit.previewData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-preview-input-invoice></cit-preview-input-invoice>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.previewData.customInvoice',
                url: '/customInvoice',
                views: {
                    '@cit.previewData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-preview-custom-invoice-sheet></cit-preview-custom-invoice-sheet>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.reductionData',
                url: '/reductionData',
                views: {
                    '@cit': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-import-layout></cit-import-layout>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.reductionData.accountMapping',
                url: '/accountMapping',
                views: {
                    '@cit.reductionData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-account-mapping></cit-account-mapping>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.reductionData.goodsMapping',
                url: '/goodsMapping',
                views: {
                    '@cit.reductionData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-goods-mapping></cit-goods-mapping>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.reductionData.caculateData',
                url: '/caculateData',
                views: {
                    '@cit.reductionData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-caculate-data></cit-caculate-data>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.reductionData.inputInvoice',
                url: '/inputInvoice',
                views: {
                    '@cit.reductionData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<p>inputInvoice</p>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.reductionData.unbilledInvoice',
                url: '/unbilledInvoice',
                views: {
                    '@cit.reductionData': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-unbilled-invoice></cit-unbilled-invoice>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });


            $stateProvider.state({
                name: 'cit.generateReport',
                url: '/generateReport',
                views: {
                    '@cit': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-report-layout></cit-report-layout>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: false,
                sticky: false
            });

            $stateProvider.state({
                name: 'cit.generateReport.reportView',
                url: '/reportView?{id:string}&{templateid:string}&{name:string}&{templatecode:string}&{initrow:any}&{initcol:any}&{parentviewid:string}',
                views: {
                    '@cit.generateReport': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            $scope.reportId = $stateParams.id;
                                            $scope.templateId = $stateParams.templateid;
                                            $scope.templateName = $stateParams.name;
                                            $scope.templateCode = $stateParams.templatecode;
                                            $scope.initRow = $stateParams.initrow;
                                            $scope.initCol = $stateParams.initcol;
                                            $scope.parentViewId = $stateParams.parentviewid;
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<vat-report-view report-id="reportId" template-id="templateId" template-name="templateName" template-code="templateCode" \
init-row="initRow" init-col="initCol" service-type="\'6\'" show-col="false" is-read-only="true" parent-view-id="parentViewId"></vat-report-view>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: false,
                sticky: false
            });

            // TODO: DCH Demo
            $stateProvider.state({
                name: 'cit.generateReport.performOCR',
                url: '/performOCR',
                views: {
                    '@cit.generateReport': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            $scope.demoFrameOptions = {};
                                            $scope.demoFrameApi = {};
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<demo-frame is-show="true" demo-frame-options="demoFrameOptions" demo-review-notes-api="demoFrameApi"></demo-frame>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: false,
                sticky: false
            });

            $stateProvider.state({
                name: 'cit.analyzeLayout',
                url: '/citAnalyzeLayout',
                views: {
                    '@cit': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.vat, appTranslation.cit]);
                                        }],
                        template: '<cit-analysis-menu></cit-analysis-menu>',

                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.analyzeLayout.analyzeReport',
                url: '/citAnalyzeReport',
                views: {
                    '@cit.analyzeLayout': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.vat, appTranslation.cit]);
                                        }],
                        template: '<cit-model-analysis></cit-model-analysis>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            $stateProvider.state({
                name: 'cit.analyzeLayout.organizeDashboard',
                url: '/citOrganizeDashboard',
                views: {
                    '@cit.analyzeLayout': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-organization-dashboard></cit-organization-dashboard>',

                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

            //$stateProvider.state({
            //    name: 'cit.analyzeReport',
            //    url: '/citAnalyzeReport',
            //    views: {
            //        '@cit': {
            //            controller: ['$scope', '$stateParams', 'appTranslation',
            //                            function ($scope, $stateParams, appTranslation) {
            //                                appTranslation.load([appTranslation.vat, appTranslation.cit]);
            //                            }],
            //            template: '<cit-model-analysis></cit-model-analysis>',
            //        }
            //    },
            //    resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
            //    deepStateRedirect: true,
            //    sticky: true
            //});

            $stateProvider.state({
                name: 'cit.analyzeReport.citModelAnalysis',
                url: '/citModelAnalysis',
                views: {
                    '@cit': {
                        controller: ['$scope', '$stateParams', 'appTranslation',
                                        function ($scope, $stateParams, appTranslation) {
                                            appTranslation.load([appTranslation.cit]);
                                        }],
                        template: '<cit-model-analysis></cit-model-analysis>',
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.cit),
                deepStateRedirect: true,
                sticky: true
            });

        }]);

var dataImpModule = angular.module('app.dataImp', ['ui.grid', 'ui.router','ui.grid.selection', 'ui.grid.moveColumns', 'ui.grid.grouping', 'ui.grid.expandable', 'ui.grid.treeView', 'ui.grid.exporter', 'ui.grid.resizeColumns', 'mc.resizer', 'ui.select', 'ngAnimate', 'ngSanitize', 'ui.bootstrap', 'ui.grid.edit', 'perfect_scrollbar', 'dx', 'ui.grid.pinning','ngFileSaver'])
    .run(['$log', function ($log) {
        $log.debug('app.dataImp.run()...');
    }])
    .config(['$controllerProvider', '$urlRouterProvider', '$compileProvider', '$filterProvider', '$provide', '$stateProvider', 'uiSelectConfig', 'scriptDependencyProvider',
        function ($controllerProvider, $urlRouterProvider, $compileProvider, $filterProvider, $provide, $stateProvider, uiSelectConfig, scriptDependencyProvider) {
            'use strict';

            // this is required to add controller/directive/filter/service after angular bootstrap
            bindModule(dataImpModule, $controllerProvider, $compileProvider, $filterProvider, $provide);

            uiSelectConfig.theme = 'select2';

            /*$stateProvider.state('dataImport.main', {
                url: "/main",
                sticky: true,
                dsr: true,
                views: {
                    'main-view@dataImport': {
                        template: '<dataImp-layout></dataImp-layout>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.dataImp)
            });*/

            $stateProvider.state({
                name: 'financialDataExtraction',
                url: "/dataExtraction/financial",
                dsr: true,
                views: {
                    'importContent': {
                        controller: ['$scope', '$state','appTranslation',
                            function ($scope, $state, appTranslation) {
                                $scope.state = $state;
                                appTranslation.load([appTranslation.appPart]);
                            }],
                        template: '<financial-data-extraction></financial-data-extraction>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.dataImp),
                sticky: true
            });

            $stateProvider.state({
                name: 'profitLossImportDistribution',
                url: "/importDistribution/profitLoss",
                dsr: true,
                views: {
                    'importContent': {
                        controller: ['$scope', '$state','appTranslation',
                            function ($scope, $state, appTranslation) {
                                $scope.state = $state;
                                appTranslation.load([appTranslation.appPart]);
                            }],
                        template: '<data-import-profit-loss></data-import-profit-loss>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.dataImp),
                sticky: true
            });

            $stateProvider.state({
                name: 'balanceSheetImportDistribution',
                url: "/importDistribution/balanceSheet",
                dsr: true,
                views: {
                    'importContent': {
                        controller: ['$scope', '$state','appTranslation',
                            function ($scope, $state, appTranslation) {
                                $scope.state = $state;
                                appTranslation.load([appTranslation.appPart]);
                            }],
                        template: '<data-import-balance-sheet></data-import-balance-sheet>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.dataImp),
                sticky: true
            });

            $stateProvider.state({
                name: 'adjustmentTableImportDistribution',
                url: "/importDistribution/adjustmentTable",
                dsr: true,
                views: {
                    'importContent': {
                        controller: ['$scope', '$state','appTranslation',
                            function ($scope, $state, appTranslation) {
                                $scope.state = $state;
                                appTranslation.load([appTranslation.appPart]);
                            }],
                        template: '<data-import-adjustment-table></data-import-adjustment-table>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.dataImp),
                sticky: true
            });

            $stateProvider.state({
                name: 'coupaPurchasingReportImportDistribution',
                url: "/importDistribution/coupaPurchasingReport",
                dsr: true,
                views: {
                    'importContent': {
                        controller: ['$scope', '$state','appTranslation',
                            function ($scope, $state, appTranslation) {
                                $scope.state = $state;
                                appTranslation.load([appTranslation.appPart]);
                            }],
                        template: '<data-import-coupa-purchasing-report></data-import-coupa-purchasing-report>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.dataImp),
                sticky: true
            });

            $stateProvider.state({
                name: 'cashFlowImportDistribution',
                url: "/importDistribution/cashFlow",
                dsr: true,
                views: {
                    'importContent': {
                        controller: ['$scope', '$state','appTranslation',
                            function ($scope, $state, appTranslation) {
                                $scope.state = $state;
                                appTranslation.load([appTranslation.appPart]);
                            }],
                        template: '<data-import-cash-flow></data-import-cash-flow>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.dataImp),
                sticky: true
            });

            $stateProvider.state({
                name: 'VATInvoiceRecordImportDistribution',
                url: "/importDistribution/VATInvoiceRecord",
                dsr: true,
                views: {
                    'importContent': {
                        controller: ['$scope', '$state','appTranslation',
                            function ($scope, $state, appTranslation) {
                                $scope.state = $state;
                                appTranslation.load([appTranslation.appPart]);
                            }],
                        template: '<data-import-invoice-record></data-import-invoice-record>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.dataImp),
                sticky: true
            });

            $stateProvider.state({
                name: 'redLetterInfoTableImportDistribution',
                url: "/importDistribution/redLetterInfoTable",
                dsr: true,
                views: {
                    'importContent': {
                        controller: ['$scope', '$state','appTranslation',
                            function ($scope, $state, appTranslation) {
                                $scope.state = $state;
                                appTranslation.load([appTranslation.appPart]);
                            }],
                        template: '<data-import-red-letter-info-table></data-import-red-letter-info-table>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.dataImp),
                sticky: true
            });

            $stateProvider.state({
                name: 'invoiceDataImportDistribution',
                url: "/importDistribution/invoiceData",
                dsr: true,
                views: {
                    'importContent': {
                        controller: ['$scope', '$state','appTranslation',
                            function ($scope, $state, appTranslation) {
                                $scope.state = $state;
                                appTranslation.load([appTranslation.appPart]);
                            }],
                        template: '<data-import-invoice-data></data-import-invoice-data>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.dataImp),
                sticky: true
            });

            $stateProvider.state({
                name: 'citJournalAdjustImportDistribution',
                url: "/importDistribution/citJournalAdjust",
                dsr: true,
                views: {
                    'importContent': {
                        controller: ['$scope', '$state','appTranslation',
                            function ($scope, $state, appTranslation) {
                                $scope.state = $state;
                                appTranslation.load([appTranslation.appPart]);
                            }],
                        template: '<data-import-cit-journal-adjust></data-import-cit-journal-adjust>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.dataImp),
                sticky: true
            });

            $stateProvider.state({
                name: 'recordImportLog',
                url: "/dataImportLog/dataImport",
                dsr: true,
                views: {
                    'importContent': {
                        controller: ['$scope', '$state','appTranslation',
                            function ($scope, $state, appTranslation) {
                                $scope.state = $state;
                                appTranslation.load([appTranslation.appPart]);
                            }],
                        template: '<import-log></import-log>'
                    }
                },
                resolve: scriptDependencyProvider.createDependenciesMap(scriptDependencyProvider.dataImp),
                sticky: true
            });

        }]);